import { nativeDecimals, chainToPlatform, } from "@wormhole-foundation/connect-sdk";
import { IBC_TRANSFER_PORT, chainToNativeDenoms, cosmwasmChainIdToNetworkChainPair, } from "./constants";
import { CosmwasmPlatform } from "./platform";
import { CosmwasmAddress } from "./address";
// forces CosmwasmUtils to implement PlatformUtils
var _ = CosmwasmUtils;
/**
 * @category CosmWasm
 */
// Provides runtime concrete value
export var CosmwasmUtils;
(function (CosmwasmUtils) {
    function nativeTokenId(chain) {
        if (!isSupportedChain(chain))
            throw new Error(`invalid chain for CosmWasm: ${chain}`);
        return {
            chain: chain,
            // TODO
            // @ts-ignore
            address: new CosmwasmAddress(getNativeDenom(chain)),
        };
    }
    CosmwasmUtils.nativeTokenId = nativeTokenId;
    function isSupportedChain(chain) {
        const platform = chainToPlatform(chain);
        return platform === CosmwasmPlatform.platform;
    }
    CosmwasmUtils.isSupportedChain = isSupportedChain;
    function isNativeTokenId(chain, tokenId) {
        if (!isSupportedChain(chain))
            return false;
        if (tokenId.chain !== chain)
            return false;
        const native = nativeTokenId(chain);
        return native == tokenId;
    }
    CosmwasmUtils.isNativeTokenId = isNativeTokenId;
    async function getDecimals(chain, rpc, token) {
        if (token === "native")
            return nativeDecimals(CosmwasmPlatform.platform);
        const addrStr = new CosmwasmAddress(token).toString();
        const { decimals } = await rpc.queryContractSmart(addrStr, {
            token_info: {},
        });
        return decimals;
    }
    CosmwasmUtils.getDecimals = getDecimals;
    async function getBalance(chain, rpc, walletAddress, token) {
        if (token === "native") {
            const { amount } = await rpc.getBalance(walletAddress, getNativeDenom(chain));
            return BigInt(amount);
        }
        const addrStr = new CosmwasmAddress(token).toString();
        const { amount } = await rpc.getBalance(walletAddress, addrStr);
        return BigInt(amount);
    }
    CosmwasmUtils.getBalance = getBalance;
    async function getBalances(chain, rpc, walletAddress, tokens) {
        const client = CosmwasmPlatform.getQueryClient(rpc);
        const allBalances = await client.bank.allBalances(walletAddress);
        const balancesArr = tokens.map((token) => {
            const address = token === "native"
                ? getNativeDenom(chain)
                : new CosmwasmAddress(token).toString();
            const balance = allBalances.find((balance) => balance.denom === address);
            const balanceBigInt = balance ? BigInt(balance.amount) : null;
            return { [address]: balanceBigInt };
        });
        return balancesArr.reduce((obj, item) => Object.assign(obj, item), {});
    }
    CosmwasmUtils.getBalances = getBalances;
    function getNativeDenom(chain) {
        // TODO: required because of const map
        if (CosmwasmPlatform.network === "Devnet")
            throw new Error("No devnet native denoms");
        return chainToNativeDenoms(CosmwasmPlatform.network, chain);
    }
    function isNativeDenom(chain, denom) {
        return denom === getNativeDenom(chain);
    }
    CosmwasmUtils.isNativeDenom = isNativeDenom;
    async function sendWait(chain, rpc, stxns) {
        const txhashes = [];
        for (const stxn of stxns) {
            const result = await rpc.broadcastTx(stxn);
            if (result.code !== 0)
                throw new Error(`Error sending transaction (${result.transactionHash}): ${result.rawLog}`);
            txhashes.push(result.transactionHash);
        }
        return txhashes;
    }
    CosmwasmUtils.sendWait = sendWait;
    async function getCurrentBlock(rpc) {
        return rpc.getHeight();
    }
    CosmwasmUtils.getCurrentBlock = getCurrentBlock;
    function chainFromChainId(chainMoniker) {
        const networkChainPair = cosmwasmChainIdToNetworkChainPair.get(chainMoniker);
        if (networkChainPair === undefined)
            throw new Error(`Unknown Cosmwasm chainId ${chainMoniker}`);
        const [network, chain] = networkChainPair;
        return [network, chain];
    }
    CosmwasmUtils.chainFromChainId = chainFromChainId;
    async function chainFromRpc(rpc) {
        const chainId = await rpc.getChainId();
        return chainFromChainId(chainId);
    }
    CosmwasmUtils.chainFromRpc = chainFromRpc;
    async function getCounterpartyChannel(sourceChannel, rpc) {
        const queryClient = CosmwasmPlatform.getQueryClient(rpc);
        const conn = await queryClient.ibc.channel.channel(IBC_TRANSFER_PORT, sourceChannel);
        return conn.channel?.counterparty?.channelId ?? null;
    }
    CosmwasmUtils.getCounterpartyChannel = getCounterpartyChannel;
})(CosmwasmUtils || (CosmwasmUtils = {}));
//# sourceMappingURL=platformUtils.js.map